mosaicsRunAll <- function( chipDir=NULL, chipFileName=NULL, chipFileFormat=NULL, 
    controlDir=NULL, controlFileName=NULL, controlFileFormat=NULL, 
    binfileDir=NULL, peakDir=NULL, peakFileName=NULL, peakFileFormat=NULL,
    reportSummary=FALSE, summaryDir=NULL, summaryFileName=NULL, 
    reportExploratory=FALSE, exploratoryDir=NULL, exploratoryFileName=NULL, 
    reportGOF=FALSE, gofDir=NULL, gofFileName=NULL,
    FDR=0.05, fragLen=200, binSize=fragLen, capping=3, 
    analysisType="IO", d=0.25, 
    signalModel="BIC", maxgap=fragLen, minsize=50, thres=10, nCore=8 ) {
	
	# check options: input & output (required)
	
	if ( is.null(chipDir) ) { stop( "Please specify 'chipDir'!" ) }
	if ( is.null(chipFileName) ) { stop( "Please specify 'chipFileName'!" ) }
	if ( is.null(chipFileFormat) ) { stop( "Please specify 'chipFileFormat'!" ) }
	
	if ( is.null(controlDir) ) { stop( "Please specify 'controlDir'!" ) }
	if ( is.null(controlFileName) ) { stop( "Please specify 'controlFileName'!" ) }
	if ( is.null(controlFileFormat) ) { stop( "Please specify 'controlFileFormat'!" ) }
	
	if ( is.null(peakDir) ) { stop( "Please specify 'peakDir'!" ) }
  if ( is.null(peakFileName) ) { stop( "Please specify 'peakFileName'!" ) }
	if ( is.null(peakFileFormat) ) { stop( "Please specify 'peakFileFormat'!" ) }
	
	if ( is.null(binfileDir) ) { stop( "Please specify 'binfileDir'!" ) }
  
  # check options: analysisType (fixed)
	
	if ( analysisType!="IO" ) {
	 message( "Info: only input-only analysis is currently supported in 'mosaicsRunAll'." )
	 message( "Info: 'analysisType' is set to 'IO'." )
	}
	
	# check options: reports (optional)
	
	if ( reportSummary ) {
    if ( is.null(summaryDir) ) {
      stop( "Please specify 'summaryDir'!" )
    }
    if ( is.null(summaryFileName) ) {
      stop( "Please specify 'summaryFileName'!" )
    }
  }
  
  if ( reportGOF ) {
    if ( is.null(gofDir) ) {
      stop( "Please specify 'gofDir'!" )
    }
    if ( is.null(gofFileName) ) {
      stop( "Please specify 'gofFileName'!" )
    }
  }
  
  if ( reportExploratory ) {
    if ( is.null(exploratoryDir) ) {
      stop( "Please specify 'exploratoryDir'!" )
    }
    if ( is.null(exploratoryFileName) ) {
      stop( "Please specify 'exploratoryFileName'!" )
    }
  }
  
  # construction of bin-level files

	cat( "Info: constructing bin-level files...\n" )
  
  constructAll( chipDir=chipDir, chipFileName=chipFileName, chipFileFormat=chipFileFormat,
    controlDir=controlDir, controlFileName=controlFileName, controlFileFormat=controlFileFormat, binfileDir=binfileDir, fragLen=fragLen, binSize=binSize, capping=capping )
	
	# read in bin-level files                       	
	
	cat( "Info: analyzing bin-level files...\n" )
	
	currentLoc <- getwd()
	
  setwd( binfileDir )
	list_chip <- system( 
    paste("ls *_",chipFileName,"_fragL",fragLen,"_bin",binSize,".txt",sep=""), 
    intern=TRUE ) 
	list_control <- system( 
    paste("ls *_",controlFileName,"_fragL",fragLen,"_bin",binSize,".txt",sep=""), 
    intern=TRUE )
  setwd(currentLoc)
	
	# check list of chromosomes & analyze only chromosomes 
  # that bin-level files for both chip & control exist
	
	chrID_chip <- unlist( lapply( strsplit( list_chip, paste("_",chipFileName,sep="") ), 
    function(x) x[1] ) )
	chrID_control <- unlist( lapply( strsplit( list_control, paste("_",controlFileName,sep="") ), 
    function(x) x[1] ) )		
	index_chip <- which( !is.na( match( chrID_chip, chrID_control ) ) )
	index_control <- match( chrID_chip, chrID_control )
	
	# model fitting & peak calling
	# check whether rparallel is available. if so, use it.
	
	cat( "Info: fitting MOSAiCS model & call peaks...\n" )
	
	chrID_list <- c()
  bin_list <- fit_list <- peak_list <- list()
  peakSetFinal <- c()  
  n_peaks <- peak_width <- opt_sig_model <- c()
  
  if ( length(index_chip) < nCore ) {
    nCore <- length(index_chip)
  }
	
	if ("rparallel" %in% names(getLoadedDLLs())) {
		runParallel( resultVar = c("chrID_list","bin_list","fit_list","peak_list","peakSetFinal","n_peaks","peak_width","opt_sig_model"), resultOp = c("c","c","c","c","rbind","c","c","c"), nWorkers = nCore )
	} else {		
		for ( i in 1:length(index_chip) ) {
			chrID <- as.character( chrID_chip[index_chip[i]] )
			chrID_list <- c( chrID_list, chrID )
			
			# read in bin-level file

			chip_file <- list_chip[ index_chip[i] ]
			input_file <- list_control[ index_control[i] ]
			
			setwd( binfileDir )       			
      bin <- readBins( type=c("chip","input"), fileName=c(chip_file,input_file) )
      bin_list <- c( bin_list, bin )      
      setwd( currentLoc )
      
      # fit model
      
      fit <- mosaicsFit( bin, analysisType=analysisType, d=d )            
      fit_list <- c( fit_list, fit )
      
      # call peaks
      
      if ( signalModel=="BIC" ) {
        # if not specified, use BIC
        
        if ( fit@bic1S < fit@bic2S ) {
          peak <- mosaicsPeak( fit, signalModel="1S", 
            FDR=FDR, maxgap=maxgap, minsize=minsize, thres=thres )      
          opt_sig_model <- c( opt_sig_model, "One-signal-component model" )
        } else {        
          peak <- mosaicsPeak( fit, signalModel="2S", 
            FDR=FDR, maxgap=maxgap, minsize=minsize, thres=thres )      
          opt_sig_model <- c( opt_sig_model, "Two-signal-component model" )
        }
      } else {                                                  
        peak <- mosaicsPeak( fit, signalModel=signalModel, 
          FDR=FDR, maxgap=maxgap, minsize=minsize, thres=thres )
        if ( signalModel=="1S" ) {
          opt_sig_model <- c( opt_sig_model, "One-signal-component model" )
        } else {
          opt_sig_model <- c( opt_sig_model, "Two-signal-component model" )
        }
      }  
      
      # keep results
      
      peak_list <- c( peak_list, peak )
      
      peakOut <- data.frame( chrID, print(peak), stringsAsFactors=FALSE )
      colnames(peakOut)[1] <- "chrID"
      peakSetFinal <- rbind( peakSetFinal, peakOut )
      
      n_peaks <- c( n_peaks, nrow(peak@peakList) )
      peak_width <- c( peak_width, median(peak@peakList$peakSize) )      
    }
  }
  
  # write peak calling results                      
  
	cat( "Info: writing the peak list...\n" )
  
  if ( peakFileFormat=="txt" ) {
    .exportTXT( peakList=peakSetFinal, fileLoc=peakDir, fileName=peakFileName )
  } else if ( peakFileFormat=="bed" ) {
    .exportBED( peakList=peakSetFinal, fileLoc=peakDir, fileName=peakFileName )
  } else if ( peakFileFormat=="gff" ) {
    .exportGFF( peakList=peakSetFinal, fileLoc=peakDir, fileName=peakFileName )
  } else {
    stop( "Inappropriate peak file format!" )  
  }
  
  # report: summary
  
	cat( "Info: generating reports...\n" )
  
	if ( reportSummary ) {    
    setwd(summaryDir)
    
    cat( "MOSAiCS: Summary of model fitting and peak calling\n", file=summaryFileName )
    cat( "\n", file=summaryFileName, append=TRUE )
    cat( "------------------------------------------------------------\n", 
      file=summaryFileName, append=TRUE )
    cat( "Input/output file settings\n", file=summaryFileName, append=TRUE )
    cat( "------------------------------------------------------------\n", 
      file=summaryFileName, append=TRUE )
    cat( "\n", file=summaryFileName, append=TRUE )
    
  	cat( "Directory of aligned read file (ChIP):", chipDir, "\n",
      file=summaryFileName, append=TRUE )
  	cat( "Name of aligned read file (ChIP): ",chipFileName,"\n", sep="",
      file=summaryFileName, append=TRUE )
  	cat( "Aligned read file format (ChIP):", chipFileFormat, "\n",
      file=summaryFileName, append=TRUE )
        	
    cat( "\n", file=summaryFileName, append=TRUE )
    
  	cat( "Directory of aligned read file (control):", controlDir, "\n",
      file=summaryFileName, append=TRUE )
  	cat( "Name of aligned read file (control): ",controlFileName,"\n", sep="",
      file=summaryFileName, append=TRUE )
  	cat( "Aligned read file format (control):", controlFileFormat, "\n",
      file=summaryFileName, append=TRUE )
        	     
    cat( "\n", file=summaryFileName, append=TRUE )
    
  	cat( "Directory of peak result file:", peakDir, "\n",
      file=summaryFileName, append=TRUE )
  	cat( "Name of peak result file: ",peakFileName,"\n", sep="",
      file=summaryFileName, append=TRUE )
  	cat( "Peak result file format:", peakFileFormat, "\n",
      file=summaryFileName, append=TRUE )
               
    cat( "\n", file=summaryFileName, append=TRUE )    
    
    cat( "------------------------------------------------------------\n", 
      file=summaryFileName, append=TRUE )
    cat( "Parameter settings\n", file=summaryFileName, append=TRUE )
    cat( "------------------------------------------------------------\n", 
      file=summaryFileName, append=TRUE )
    cat( "\n", file=summaryFileName, append=TRUE )
    
    if ( analysisType=="OS" ) {
      analysisTypeOut <- "One-sample analysis"
    } else if ( analysisType=="TS" ) {
      analysisTypeOut <- "Two-sample analysis (with mappability & GC content)"
    } else if ( analysisType=="IO" ) {
      analysisTypeOut <- "Two-sample analysis (Input only)"
    }
    
    if ( signalModel=="BIC" ) {
      signalModelOut <- "Automatic signal model selection based on BIC"
    } else if ( signalModel=="1S" ) {
      signalModelOut <- "One-signal-component model"
    } else if ( signalModel=="2S" ) {
      signalModelOut <- "Two-signal-component model"
    }
      
  	cat( "False discovery rate (FDR):", FDR, "\n", file=summaryFileName, append=TRUE )
    cat( "Fragment length:", fragLen, "\n", file=summaryFileName, append=TRUE )
  	cat( "Bin size:", binSize, "\n", file=summaryFileName, append=TRUE )
  	cat( "Maximum number of reads allowed in each nucleotide:", capping, "\n",
      file=summaryFileName, append=TRUE )
    cat( "Analysis type:", analysisTypeOut, "\n", file=summaryFileName, append=TRUE )
    cat( "d:", d, "\n", file=summaryFileName, append=TRUE )
    cat( "Signal model:", signalModelOut, "\n", file=summaryFileName, append=TRUE )
    cat( "maxgap:", maxgap, "\n", file=summaryFileName, append=TRUE )
    cat( "minsize:", minsize, "\n", file=summaryFileName, append=TRUE )
    cat( "thres:", thres, "\n", file=summaryFileName, append=TRUE )
       
    cat( "\n", file=summaryFileName, append=TRUE )
    cat( "------------------------------------------------------------\n", 
      file=summaryFileName, append=TRUE )
    cat( "Peak calling summary\n", file=summaryFileName, append=TRUE )
    cat( "------------------------------------------------------------\n", 
      file=summaryFileName, append=TRUE )
    cat( "\n", file=summaryFileName, append=TRUE )
    
    outFormat <- data.frame( chrID_list, n_peaks, peak_width, opt_sig_model,
      stringsAsFactors=FALSE )
    colnames(outFormat) <- c( "chrID", "# peaks", "Median peak width", "Optimal/specified signal model" )
      
    cat( as.character(colnames(outFormat)), file=summaryFileName, sep="\t", append=TRUE )
    cat( "\n", file=summaryFileName, append=TRUE )
    cat( rep("-----",3), file=summaryFileName, sep="\t", append=TRUE )
    cat( "\t\t\t-----", file=summaryFileName, sep="\t", append=TRUE )
    cat( "\n", file=summaryFileName, append=TRUE )
    
    # peak list
     
    for ( i in 1:nrow(outFormat) )
    {
        cat( as.character(outFormat[i,])[1:3], file=summaryFileName, sep="\t", append=TRUE )
        cat( "\t\t", as.character(outFormat[i,])[4], 
          file=summaryFileName, sep="\t", append=TRUE )
        cat( "\n", file=summaryFileName, append=TRUE )
    }
            
    setwd(currentLoc)
	}
  		
	# GOF
  
  if ( reportGOF ) {    
    setwd(gofDir)
    
    pdf(gofFileName)
    for ( i in 1:length(chrID_list) ) {
  		chrID <- chrID_list[i]
  		fit <- fit_list[[i]]  		
  		
  		# chrID
      
      plot( 0, 0, type="n", axes=F, ann=F )
  	  text( 0, 0, chrID, cex=4 )
  	  
  		# GOF
      
      plot(fit)
    }
    dev.off()
    
    setwd(currentLoc)
  }
  		
	# exploratory analysis
  
  if ( reportExploratory ) {
    setwd(exploratoryDir)
    
    pdf(exploratoryFileName)
    for ( i in 1:length(chrID_list) ) {
  		chrID <- chrID_list[i]
  		bin <- bin_list[[i]]
  		
  		# chrID
      
      plot( 0, 0, type="n", axes=F, ann=F )
  	  text( 0, 0, chrID, cex=4 )
  	  
  		# exploratory plots 
      
      plot( bin )
  		plot( bin, plotType="input" )
  		if ( analysisType=="OS" || analysisType=="TS" ) {
    		plot( bin, plotType="M" )  		
    		plot( bin, plotType="GC" )  		
    		plot( bin, plotType="M|input" )  		
    		plot( bin, plotType="GC|input" )  		
  		}
    }
    dev.off()
    
    setwd(currentLoc)
  }
}

constructAll <- function( chipDir, chipFileName, chipFileFormat, controlDir, controlFileName, controlFileFormat, binfileDir, fragLen, binSize, capping ) {
  # stack for rparallel
  
  dir_list <- c( chipDir, controlDir )
	file_list <- c( chipFileName, controlFileName ) 
	format_list <- c( chipFileFormat, controlFileFormat )
	
	# process aligned read files into bin-level files
  	
	proc_list <- c()
	
	if ("rparallel" %in% names(getLoadedDLLs())) {
		runParallel( resultVar = "proc_list", resultOp = "c", nWorkers = length(file_list) )
	} else {		
    for ( i in 1:length(file_list) ) {
      constructBins( infileLoc = dir_list[i], infileName = file_list[i], fileFormat = format_list[i], 
        outfileLoc = binfileDir, fragLen = fragLen, binSize = binSize, capping = capping )
      proc_list <- c( proc_list, file_list[i] )
    }
  }
}